/*
  Copyright (C) 2000-2005 SKYRIX Software AG

  This file is part of SOPE.

  SOPE is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  SOPE is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with SOPE; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#include "common.h"
#import <objc/objc.h>
#import <objc/Protocol.h>

#if NeXT_RUNTIME || APPLE_RUNTIME
#  import <objc/objc.h>
#  import <objc/objc-class.h>
#endif

@implementation NSObject(Reflection)

+ (void)_addSelectorsOfClassToArray:(NSMutableSet *)_sels {
  unsigned int i;
  unsigned int methodCount = 0;
  
  NSLog(@"adding selectors of class: %@", NSStringFromClass(self));
  Method * mlist = class_copyMethodList(self, &methodCount);
  
  for ( i=0; i<methodCount; i++ ) {
      const char * selName = sel_getName(method_getName(mlist[i]));
      NSString * sName = [[NSString alloc] initWithBytesNoCopy: (char*)selName
	length: strlen(selName) encoding: NSUTF8StringEncoding freeWhenDone: NO];
      //NSLog(@"  processing %i selectors ...", mlist->method_count);

      if (strlen(selName) == 0) {
        NSLog(@"WARNING(%s): did not get selector for method 0x%p",
              __PRETTY_FUNCTION__, mlist[i]);
        continue;
      [_sels addObject:sName];
      [sName release];
    }
  }
  free(mlist);
}

+ (NSArray *)classImplementsSelectors {
  NSMutableSet *a;
  
  a = [[[NSMutableSet alloc] initWithCapacity:32] autorelease];
  [self _addSelectorsOfClassToArray:a];
  return [a allObjects];
}

+ (NSArray *)instancesRespondToSelectors {
  NSMutableSet *a;
  Class clazz;
  
  a = [NSMutableSet setWithCapacity:128];
  for (clazz = self; clazz; clazz = [clazz superclass])
    [clazz _addSelectorsOfClassToArray:a];
  return [a allObjects];
}

- (NSArray *)respondsToSelectors {
  return [[self class] instancesRespondToSelectors];
}

@end /* NSObject(Reflection) */
